import matplotlib.pyplot as plt
import matplotlib as mpl

mpl.rcParams["axes.titlesize"] = "medium"
mpl.rcParams["axes.grid"] = True
mpl.rcParams["axes.formatter.use_mathtext"] = True
mpl.rcParams["xtick.top"] = True
mpl.rcParams["xtick.labelsize"] = "small"
mpl.rcParams["xtick.direction"] = "in"
mpl.rcParams["xtick.minor.visible"] = True
mpl.rcParams["ytick.right"] = True
mpl.rcParams["ytick.labelsize"] = "small"
mpl.rcParams["ytick.direction"] = "in"
mpl.rcParams["ytick.minor.visible"] = True
mpl.rcParams["grid.linewidth"] = 0.5
mpl.rcParams["legend.fontsize"] = "small"
mpl.rcParams["legend.framealpha"] = 1.0

print("aiken_plot_defaults.py imported.")


def plot(x, y, x_label=None, y_label=None, title=None, c="black", hline=None):
    plt.plot(x, y, c=c)
    _plot_options(x=x, x_label=x_label, y_label=y_label, title=title, hline=hline)
    plt.show()


def scatter(x, y, x_label=None, y_label=None, title=None, s=1, c="black", hline=None):
    plt.scatter(x, y, s=s, c=c)
    _plot_options(x=x, x_label=x_label, y_label=y_label, title=title, hline=hline)
    plt.show()


def plot_series(x, y: dict, x_label=None, y_label=None, title=None, hline=None):
    for k in y:
        plt.plot(x, y[k], label=k)
    _plot_options(x=x, x_label=x_label, y_label=y_label, title=title, hline=hline)
    plt.legend()
    plt.show()


def _plot_options(x, x_label=None, y_label=None, title=None, hline=None):
    if hline:
        plt.hlines(hline, x.min(), x.max(), linestyles="dashed")
    if x_label:
        plt.xlabel(x_label)
    if y_label:
        plt.ylabel(y_label)
    if title:
        plt.title(title)


def plot_2d_scatter_colorbar(data, title=None, figure_size=(20, 5), color_map="hsv"):
    """
    Plot a 2d scatter plot with colorbar.
    data is a dictionary with keys 'x', 'y' and 'z'
    """
    x, y, z = data.keys()
    fig = plt.figure(figsize=figure_size)
    ax = fig.add_subplot(111)
    p = ax.scatter(data[x], data[y], s=2, c=data[z], cmap=color_map)
    _plot_options(x=None, x_label=x, y_label=y, title=title, hline=None)
    fig.colorbar(p, ax=ax, shrink=0.75, label=z)
    plt.show()



"""
These calculations are based on the Aiken impact model.  If you know... you know...
"""

from pathlib import Path

import matplotlib.pyplot as plt
import numpy as np
import pandas as pd
import seaborn as sns


def import_plate_fea(
    path: Path,
    thickness_list: list[int],
    diameter_list: list[int],
    material_list: list[str] = ["a572"],
) -> pd.DataFrame:
    """
    Import plate fea from the csv file repository.  Returns Pandas Dataframe
    """

    new_run = True

    for diam in diameter_list:
        for thick in thickness_list:
            for matl in material_list:
                name_string: str = str(diam) + "-" + str(thick) + "-" + matl + "-"

                filename: Path = path / (name_string + "d.txt")
                case_data: pd.DataFrame = pd.read_table(filename, sep="\t")
                case_data.drop(case_data.columns[[0, 2, 5]], axis=1, inplace=True)
                case_data.columns = [
                    "time",
                    "maximum_deformation",
                    "average_deformation",
                ]
                new_row: list[float] = [0.0, 0.0, 0.0]
                case_data.loc[1:] = case_data.loc[:]
                case_data.loc[0] = new_row

                filename: Path = path / (name_string + "e.txt")
                energy_data: pd.DataFrame = pd.read_table(filename, sep="\t")
                energy_data.drop(
                    energy_data.columns[[0, 2, 3, 5]], axis=1, inplace=True
                )
                energy_data.columns = ["time", "energy"]
                new_row: list[float] = [0.0, 0.0]
                energy_data.loc[1:] = energy_data.loc[:]
                energy_data.loc[0] = new_row

                filename: Path = path / (name_string + "s.txt")
                stress_data: pd.DataFrame = pd.read_table(filename, sep="\t")
                stress_data.drop(
                    stress_data.columns[[0, 2, 4, 5]], axis=1, inplace=True
                )
                stress_data.columns = ["time", "edge_stress"]
                new_row: list[float] = [0.0, 0.0]
                stress_data.loc[1:] = stress_data.loc[:]
                stress_data.loc[0] = new_row

                filename: Path = path / (name_string + "f1.txt")
                force1_data: pd.DataFrame = pd.read_table(filename, sep="\t")
                if len(force1_data.columns) == 7:
                    force1_data.drop(force1_data.columns[[0, 6]], axis=1, inplace=True)
                elif len(force1_data.columns) == 6:
                    force1_data.drop(force1_data.columns[[0]], axis=1, inplace=True)
                force1_data.columns = [
                    "time",
                    "perimeter_fx",
                    "perimeter_fy",
                    "perimeter_fz",
                    "perimeter_f_total",
                ]
                new_row: list[float] = [0.0, 0.0, 0.0, 0.0, 0.0]
                force1_data.loc[1:] = force1_data.loc[:]
                force1_data.loc[0] = new_row

                filename: Path = path / (name_string + "f2.txt")
                force2_data: pd.DataFrame = pd.read_table(filename, sep="\t")
                if len(force2_data.columns) == 7:
                    force2_data.drop(force2_data.columns[[0, 6]], axis=1, inplace=True)
                if len(force2_data.columns) == 6:
                    force2_data.drop(force2_data.columns[[0]], axis=1, inplace=True)
                force2_data.columns = [
                    "time",
                    "edge_fx",
                    "edge_fy",
                    "edge_fz",
                    "edge_f_total",
                ]
                new_row: list[float] = [0.0, 0.0, 0.0, 0.0, 0.0]
                force2_data.loc[1:] = force2_data.loc[:]
                force2_data.loc[0] = new_row

                total_force: pd.Series = (
                    force1_data.perimeter_f_total + force2_data.edge_f_total
                )
                edge_stress: pd.Series = (
                    np.abs(force2_data.edge_fx) / (3 * diam / 1000) / (thick / 1000)
                )

                case_data = pd.merge(case_data, energy_data)
                case_data = pd.merge(case_data, stress_data)
                case_data = pd.merge(case_data, force1_data)
                case_data = pd.merge(case_data, force2_data)

                diameter: list[float] = [diam / 1000 for _ in range(case_data.shape[0])]
                case_data.insert(1, "diameter", diameter)
                thickness: list[float] = [
                    thick / 1000 for _ in range(case_data.shape[0])
                ]
                case_data.insert(2, "thickness", thickness)
                material: list[str] = [matl for _ in range(case_data.shape[0])]
                case_data.insert(3, "material", material)
                # drops the last row for each case which probably includes some NaN
                case_data.drop(index=case_data.index[-1], axis=0, inplace=True)
                case_data.insert(7, "total_force", total_force)
                case_data.insert(8, "avg_edge_stress", edge_stress)
                if new_run == True:
                    fea_data = case_data
                    new_run = False
                else:
                    fea_data = pd.concat([fea_data, case_data])

    return fea_data


def plot_plate_fea(fea_data: pd.DataFrame, thickness: int):
    diameter_list: list[float] = fea_data.diameter.unique()

    colors = sns.color_palette("mako_r", len(diameter_list))

    fig, axes = plt.subplots(1, 3, figsize=(15, 5))
    for di, diameter in enumerate(diameter_list):
        filt = (fea_data["diameter"] == diameter) & (
            fea_data["thickness"] == thickness / 1000
        )
        ax = axes[0]
        ax.plot(
            fea_data.loc[filt, "energy"] / 1000,
            fea_data.loc[filt, "edge_f_total"] / 1000,
            "o--",
            markersize=2,
            linewidth=1,
            label=str(diameter * 1000) + "mm",
            color=colors[di],
        )

        ax = axes[1]
        ax.plot(
            fea_data.loc[filt, "energy"] / 1000,
            fea_data.loc[filt, "avg_edge_stress"] / 1e6,
            "o--",
            markersize=2,
            linewidth=1,
            color=colors[di],
        )

        ax = axes[2]
        ax.plot(
            fea_data.loc[filt, "energy"] / 1000,
            fea_data.loc[filt, "maximum_deformation"] * 1000,
            "o--",
            markersize=2,
            linewidth=1,
            color=colors[di],
        )

    ax = axes[0]
    ax.set_ylabel("Impact Force (kN)", fontsize=14)
    ax.legend(fontsize="small", title="Projectile Diameter")

    ax = axes[1]
    ax.set_ylabel("Edge Stress (MPa)", fontsize=14)
    ax.set_xlabel("Energy (kJ)", fontsize=14)
    ax.set_title(str(thickness) + " mm Plate", loc="center", fontsize=14)

    ax = axes[2]
    ax.set_ylabel("Max Displacement (mm)", fontsize=14)

    title_string = "FEA Impact Results for Single Layer " + str(thickness) + "mm Plate"
    fig.suptitle(title_string, fontsize=16)
    fig.tight_layout()

    plt.show()



"""
This file uses all the scipy.constants units and defines some additional units
that make documentation / implementation easier to follow.  For example, m=1.
Trigger Warning! - This file uses from `scipy.constants import *`
"""

from scipy.constants import *

# define some additional common units
m = 1.0
N = 1.0
s = 1.0
kg = 1.0
J = 1.0
kJ = 1000
mm = 1 / 1000 * m
cm = 1/ 100 * m
ksi = 1000 * psi
MPa = N / mm ** 2
kPa = 1000 * N / m ** 2
kN = 1000 * N
ft = 12 * inch
psf = 144 * psi
kip = 1000 * pound_force
L = .001 * m ** 3



from pathlib import Path


def read_nodes(filename: str | Path) -> dict:
    """
    Reads the NLIST file from ANSYS APDL and returns a dictionary of the nodes.  This can be useful to create a
    pandas.Dataframe.
    """
    path = Path(filename)
    nodes = {}

    with path.open(mode="r", encoding="utf-8") as f:
        content = f.readlines()

    node = []
    x = []
    y = []
    z = []

    for line in content:
        if len(line.split()) > 0:
            if (line.split()[0]).isnumeric():
                node.append(int(line.split()[0]))
                x.append(float(line.split()[1]))
                y.append(float(line.split()[2]))
                z.append(float(line.split()[3]))

    nodes["NODE"] = node
    nodes["X"] = x
    nodes["Y"] = y
    nodes["Z"] = z

    return nodes


def read_prnsol(filename: str | Path) -> dict:
    """
    Reads the PRNSOL output and returns a dict {NODE, KEY} where KEY is
    the value of the output to the prnsol file.
    """
    path = Path(filename)
    nodes = {}

    with path.open(mode="r", encoding="utf-8") as f:
        content = f.readlines()

    node = []
    ux = []
    key = None

    for line in content:
        if len(line.split()) > 0:
            if line.split()[0] == "NODE" and key is None:
                key = line.split()[1]
            if (line.split()[0]).isnumeric():
                node.append(int(line.split()[0]))
                ux.append(float(line.split()[1]))

    nodes["NODE"] = node
    nodes[key] = ux

    return nodes


def read_pretab(filename: str | Path) -> dict:
    """
    Reads the PRETAB output and returns a dict {ELEMENT, KEY} where KEY is
    the value of the output to the PRETAB file.
    """
    path = Path(filename)
    elements = {}

    with path.open(mode="r", encoding="utf-8") as f:
        content = f.readlines()

    for line in content:
        if len(line.split()) > 0:
            if line.split()[0] == "ELEM" and len(elements.keys()) == 0:
                keys = line.split()
                for key in keys:
                    elements[key] = []
            if (line.split()[0]).isnumeric():
                for i, key in enumerate(keys):
                    if key == "ELEM":
                        elements[key].append(int(line.split()[i]))
                    else:
                        elements[key].append(float(line.split()[i]))

    return elements


def read_elements(filename: str | Path) -> dict:
    """Returns a dict {ELEM, NODE1, NODE2} for beam elements."""
    path = Path(filename)
    elements = {}

    with path.open(mode="r", encoding="utf-8") as f:
        content = f.readlines()

    elem = []
    node_1 = []
    node_2 = []

    for line in content:
        if len(line.split()) > 0:
            if (line.split()[0]).isnumeric():
                elem.append(int(line.split()[0]))
                node_1.append(int(line.split()[6]))
                node_2.append(int(line.split()[7]))

    elements["ELEM"] = elem
    elements["NODE1"] = node_1
    elements["NODE2"] = node_2

    return elements

def read_linearized_stress(filename: str) -> dict:
    """This file reads the linearized stress output from ANSYS and returns a dict."""
    path = Path(filename)
    results = {}
    with path.open(mode="r", encoding="utf-8") as f:
        content = f.readlines()

    results["path"] = content[1].split()[9]

    if content[3].split()[0] == "RADIUS":
        addlines = 0
        results["radius"] = float(content[3].split()[4])
        results["inside node"] = int(content[6].split()[3])
        results["outside node"] = int(content[6].split()[7])

    else:
        addlines = -3
        results["inside node"] = int(content[4].split()[3])
        results["outside node"] = int(content[4].split()[7])

    results["SXm"] = float(content[16 + addlines].split()[0])
    results["SYm"] = float(content[16 + addlines].split()[1])
    results["SZm"] = float(content[16 + addlines].split()[2])
    results["SXYm"] = float(content[16 + addlines].split()[3])
    results["SYZm"] = float(content[16 + addlines].split()[4])
    results["SXZm"] = float(content[16 + addlines].split()[5])
    results["S1m"] = float(content[18 + addlines].split()[0])
    results["S2m"] = float(content[18 + addlines].split()[1])
    results["S3m"] = float(content[18 + addlines].split()[2])
    results["SINTm"] = float(content[18 + addlines].split()[3])
    results["SEQVm"] = float(content[18 + addlines].split()[4])
    results["SXb inside"] = float(content[22 + addlines].split()[1])
    results["SXb outside"] = float(content[24 + addlines].split()[1])
    results["SYb inside"] = float(content[22 + addlines].split()[2])
    results["SYb outside"] = float(content[24 + addlines].split()[2])
    results["SZb inside"] = float(content[22 + addlines].split()[3])
    results["SZb outside"] = float(content[24 + addlines].split()[3])
    results["SXYb inside"] = float(content[22 + addlines].split()[4])
    results["SXYb outside"] = float(content[24 + addlines].split()[4])
    results["SYZb inside"] = float(content[22 + addlines].split()[5])
    results["SYZb outside"] = float(content[24 + addlines].split()[5])
    results["S1b inside"] = float(content[26 + addlines].split()[1])
    results["S1b outside"] = float(content[28 + addlines].split()[1])
    results["S2b inside"] = float(content[26 + addlines].split()[2])
    results["S2b outside"] = float(content[28 + addlines].split()[2])
    results["S3b inside"] = float(content[26 + addlines].split()[3])
    results["S3b outside"] = float(content[28 + addlines].split()[3])
    results["SINTb inside"] = float(content[26 + addlines].split()[4])
    results["SINTb outside"] = float(content[28 + addlines].split()[4])
    results["SEQVb inside"] = float(content[26 + addlines].split()[5])
    results["SEQVb outside"] = float(content[28 + addlines].split()[5])
    results["SXp inside"] = float(content[42 + addlines].split()[1])
    results["SXp outside"] = float(content[44 + addlines].split()[1])
    results["SYp inside"] = float(content[42 + addlines].split()[2])
    results["SYp outside"] = float(content[44 + addlines].split()[2])
    results["SZp inside"] = float(content[42 + addlines].split()[3])
    results["SZp outside"] = float(content[44 + addlines].split()[3])
    results["SXYp inside"] = float(content[42 + addlines].split()[4])
    results["SXYp outside"] = float(content[44 + addlines].split()[4])
    results["SYZp inside"] = float(content[42 + addlines].split()[5])
    results["SYZp outside"] = float(content[44 + addlines].split()[5])
    results["S1p inside"] = float(content[46 + addlines].split()[1])
    results["S1p outside"] = float(content[48 + addlines].split()[1])
    results["S2p inside"] = float(content[46 + addlines].split()[2])
    results["S2p outside"] = float(content[48 + addlines].split()[2])
    results["S3p inside"] = float(content[46 + addlines].split()[3])
    results["S3p outside"] = float(content[48 + addlines].split()[3])
    results["SINTp inside"] = float(content[46 + addlines].split()[4])
    results["SINTp outside"] = float(content[48 + addlines].split()[4])
    results["SEQVp inside"] = float(content[46 + addlines].split()[5])
    results["SEQVp outside"] = float(content[48 + addlines].split()[5])
    return results